
var Emitter = require('events').EventEmitter;
var aggregate = require('..');
var knox = require('knox');

// env vars required
// to run the tests

var s3 = knox.createClient({
  key: process.env.KEY,
  secret: process.env.SECRET,
  bucket: process.env.BUCKET
})

// lame

function BufferStream() {
  this.data = new Buffer(0);
}

BufferStream.prototype.write = function(buf){
  this.data = Buffer.concat([this.data, buf]);
};

BufferStream.prototype.end = function(){
  this.ended = true;
};

/**
 * Inherit from `Emitter.prototype`.
 */

BufferStream.prototype.__proto__ = Emitter.prototype;

describe('aggregate(opts)', function(){
  before(function(done){
    s3.putBuffer('1,foo,bar,baz\n', '/test/1', function(err){
      if (err) return done(err);
      s3.putBuffer('2,foo,bar,baz\n', '/test/2', function(err){
        if (err) return done(err);
        s3.putBuffer('3,foo,bar,baz\n', '/test/3', done);
      });
    });
  })

  it('should error when required', function(done){
    var buf = new BufferStream;

    var a = aggregate({
      client: s3,
      files: ['/test/1', '/test/something', '/test/3'],
      output: buf,
      required: true
    });

    a.on('error', function(err){
      err.file.should.equal('/test/something');
      err.status.should.equal(404);
      done();
    });

    a.on('end', function(){
      done(new Error('"end" should not be emitted'));
    });
  })

  it('should ignore unless required', function(done){
    var buf = new BufferStream;

    var a = aggregate({
      client: s3,
      files: ['/test/1', '/test/something', '/test/3'],
      output: buf
    });

    a.on('error', function(err){
      done(new Error('"error" should not be emitted'));
    });

    a.on('failed', function(req){
      req.file.should.equal('/test/something');
      req.status.should.equal(404);
    });

    a.on('end', function(){
      done();
    });
  })

  it('should aggregate', function(done){
    var buf = new BufferStream;

    var a = aggregate({
      client: s3,
      files: ['/test/1', '/test/2', '/test/3'],
      output: buf,
      required: true
    });

    a.on('error', done);

    a.on('end', function(){
      var res = buf.data.toString();
      res.should.equal('1,foo,bar,baz\n2,foo,bar,baz\n3,foo,bar,baz\n');
      buf.ended.should.be.true;
      done();
    });
  })
})